
%
%  Filtered White Gaussian Noise Generator class
%

%
% function 'generate': this function returns a vector of blockSize elements that
% contains complex gain samples.
%

%
% We're overriding the parent class's 'generate' function, providing a suitable 
% implementation.
%
% NOTE: when the blockNum paramter is zero the random number generator is initialized 
% using the randomSeed member variable otherwise it is ignored because the Filterred WGN 
% generator can only generate sequentially.
%

function [c, y] = generate( c, blockNum )

% get the K factor
Kfactor = get( c, 'Kfactor' );

% init the seed for the random num gen if this is the first block being generated
if blockNum == 0
    randomSeed = get( c, 'randomSeed' );
    randn( 'state', randomSeed );
end;

% get the block size
blockSize = get( c, 'blockSize' );

% generate gaussian samples, pad with zeros and take fft
noiseSpectrum = fft( [randn( 1, blockSize )./sqrt(2) + j*randn(1, blockSize)./sqrt(2), zeros(1, blockSize) ] );

% filter the white gaussian noise samples
gainSamples = ifft( noiseSpectrum .* c.filter );

% add the first half to the previous postcursor and return the result
y = gainSamples(1:blockSize) + c.postCursor + sqrt(Kfactor/2);

% save the second half as the postcursor for the next time
c.postCursor = gainSamples(blockSize+1:2*blockSize);








