
%
% The Jakes Complex Gain Generator
%

%
% Constructor, performs initilization.
% The function has the following arguments, in the order they apear here:
%   blockSize: number of samples processed at on time
%   fDts: normalized sample spacing
%   Kfactor: the K factor used for ricean fading
%   randomSeed: the random Seed (optional)
%   numScat: number of equispaced scatterers to be simulated
%   wssThreshold: how close to +-pi/2 can the generated angles be without making
%   corThreshold: threshold value used to determine if two generators are correlated
%                 the process non-WSS
%   
function c = JakesGen( varargin )

switch nargin
case 6
    
    % get arguments
    %
    blockSize = varargin{1};
    fDts = varargin{2};
    Kfactor = varargin{3};
    randomSeed = varargin{4};
    c.numScat = varargin{5}; 
    c.wssThreshold = varargin{6}; 

case 5
    
    % get arguments
    %
    blockSize = varargin{1};
    fDts = varargin{2};
    Kfactor = varargin{3};
    randomSeed = sum(100*clock); % no random seed passed in so lets make up one
    c.numScat = varargin{4}; 
    c.wssThreshold = varargin{5}; 
    
    
case 1
    %
    % Create an object using the same properties as the object
    % passed in
    % NOTE: a new random seed is generated
    %
    
    % source object
    source = varargin{1};
    
    % get base class properties
    blockSize = get( source, 'blockSize' );
    fDts = get( source, 'fDts' );
    Kfactor = get( source, 'Kfactor' );
    randomSeed = sum(100*clock); % generate new seed
    c.numScat = source.numScat;
    c.wssThreshold = source.wssThreshold;
    
otherwise
    error( 'Wrong number of input arguments' );
end

    %
    % Initialize
    %
    
    rand( 'state', randomSeed );
    
    % random phases
    for j = [1:c.numScat]
        c.phase(j) = rand(1) * 2 * pi;
    end;
    
    % doppler shifts
    nonWSS = 1;
    while nonWSS
        
        % assume WSS
        nonWSS = 0;

        % the angle of the first scatterer
        theta0 = rand(1) * 2 * pi / c.numScat;
        
        % generate the other numScat scatterers (equispaced)
        for j = [0:c.numScat - 1]
            theta = theta0 + j * 2 * pi / c.numScat;
            
            % check WSS-ness
            if (abs(theta - pi/2) < c.wssThreshold) | (abs(theta + pi/2) < c.wssThreshold)
                nonWSS = 1;
                break;
            end;
            
            % calc doppler shift from angle
            c.dopplerShift( j + 1 ) = -2 * pi * cos( theta );
        end;
    end;

    % create base class
    gainGen = ComplexGainGen( blockSize, fDts, Kfactor, randomSeed );
    
    % create this class
    c = class( c, 'JakesGen', gainGen );


