
%
%  Jakes Complex Gain Generator class
%

%
% function 'generate': this function returns a vector of blockSize elements that
% contains complex gain samples.
%

%
% We're overriding the parent class's 'generate' function, providing a suitable 
% implementation.
%

function [c, y] = generate( c, blockNum )

    % get the K factor
    Kfactor = get( c, 'Kfactor' );

    % get block size from parent
    blockSize = get( c, 'blockSize' );    
    % ...and the normalized sample spacing
    fDts = get(c, 'fDts');
    
    % This is the time axis (sampling instants)    
    u = [blockNum*blockSize*fDts:fDts:( (blockNum+1)*blockSize - 1 ) * fDts]; 
    
    % init to zero
    y = zeros( 1, blockSize );

    % generate gain samples
    for j = [ 1 : c.numScat ]
        y = y + exp( i * ( c.phase( j ) + u * c.dopplerShift( j ) ) );
    end;
    
    % devide by sqrt( Ns ) and add the LOS
    y = y / sqrt( c.numScat ) + sqrt( Kfactor / 2 );
        






