

%
% class TDL: Tapped Delay Line
%

%
% Constructor
%       powerDelayProfile: array containing points seperated by
%       ts in the power delay profile. Each is the variance of the
%       corresponding tap in the tap delay line.
%
%       Kfactors: array of K-Factor values, one for the generator
%       at each tap.
%
%       sampleGenerator: is a sample object of the desired type
%       of complex gain generator. The needed generators will
%       created using the same properties as this one.
%
%       corThreshold: threshold value used to determine if two generators
%       are correlated.
%

function c = TDL( powerDelayProfile, Kfactors, sampleGenerator, corThreshold )
    

% save number of taps 
c.numTaps = length( powerDelayProfile );

% block size
c.blockSize = get( sampleGenerator, 'blockSize' );

% save the power delay profile
c.powerDelayProfile = powerDelayProfile;

% creat an array of uncorrelated generators
generators = createUncorGenerators( c.numTaps, sampleGenerator, corThreshold );

% Initialize the taps
for i=[1:c.numTaps]
    
    % put the generator from the array of uncorrelated gens into its tap
    % and at the same time set the k factor
    c.taps( i ).gen = set( generators( i ), 'Kfactor', Kfactors(i) );
    
    % generate the first block
    [c.taps( i ).gen, c.taps( i ).buffer] =  generate( c.taps( i ).gen, 0 );
  
%
%   Uncomment this block when using the FWN Generator to get out of the transient area
%   caused by intial zero postCursor values.
%
%     for j=1:400
%         [c.taps( i ).gen, c.taps( i ).buffer] =  generate( c.taps( i ).gen, 1 );
%     end;
end;

% delete the array of generators
clear generators;

% initialize the buffer pointer
c.gainBufferIndex = 1;

% the number of the next block 
c.blockNum = 1;

% the buffer used for convolution
c.buffer = zeros( 1, c.numTaps ); 

c = class( c, 'TDL' );


