
% generator parameters
blockSize=10000;
blockCount = 100;
fDts=0.01;
randomSeed = 1;
Kfactor = 0;
beta = 5;

% create complex gain generator
gainGen = FWNGen( blockSize, fDts, Kfactor, randomSeed, beta );

% generate 500 blocks so that we leave the transient section.
% This is because the 'postcursor' used in the FFT overlap and add
% is initially 0.
for i=1:300
    [gainGen, yTemp] = generate( gainGen, i - 1 );
end;

%
% Compare pdf with that of a Gaussian rv
%

% bins for histogram (pdf)
x = -2.9:0.1:2.9;

y = [];

% generate block
for i=1:blockCount
    [gainGen, yCurrent] = generate( gainGen, i - 1 );
    
    y = [y yCurrent];
end;

blockSize = blockSize * blockCount;

% draw pdf, making variance equal to one
figure;
pdfActual = hist(real(y)/sqrt(var(real(y))),x); 
hist(real(y)/sqrt(var(real(y))),x); 
title( 'Actual distribution' );
ylabel( 'Number of Data Samples: N_a_c_t_u_a_l(B_i)' );
xlabel( 'Bin Centers: B_i' );
V = axis;

% draw pdf of unit variance gaussian rv for comparison
figure;
pdfGaussian = hist(randn(1,blockSize),x);
hist(randn(1,blockSize),x);
title( 'Theoritical Gaussian rv' );
ylabel( 'Number of Data Samples: N_t_h_e_o_r_i_t_i_c_a_l(B_i)' );
xlabel( 'Bin Centers: B_i' );
axis(V);

% plot the difference between the actual and the theoritical pdfs
figure;
plot( x, abs( pdfActual - pdfGaussian)/blockSize );
title( 'gaussian/actual difference' );
xlabel( 'Bin Centers: B_i' );
ylabel( '|N_a_c_t_u_a_l(B_i)-N_t_h_e_o_r_i_t_i_c_a_l(B_i)|/N_t_o_t_a_l' );

% X2 test of good fit
% degFreedom = (length(pdfGaussian)-3);
% x2 = sum( (pdfActual-pdfGaussian).^2./pdfGaussian );
% p = chi2cdf( x2, degFreedom );
% 
% 
% g = normcdf( x(2:length(x)), 0, 1/2) - normcdf(x(1:length(x)-1), 0, 1/2);
% x2 = sum( ((g.*blockSize)-pdfActual(1:end-1)).^2./(g.*blockSize) );


%
% Compare the autocorrelation with theoritical -i.e. 
% zeroth order bessel function of the first kind.
%

% x axis for autocorrelatiohn
ax = -(blockSize-1)*fDts:fDts:(blockSize-1)*fDts;

% autocorrelation (normalized)
autocor = xcorr( y )/blockSize;

% bessel function
j0 = besselj(0, ax*2*pi);

% difference between actual and theoritical
diff = (real(j0 - autocor));

% plot the 400 samples in the middle
figure;
plot(ax(blockSize-10000:blockSize+10000),diff(blockSize-10000:blockSize+10000), ax(blockSize-10000:blockSize+10000), j0(blockSize-10000:blockSize+10000), '--' );

legend( 'j0 - E[ Real(y(t))*Real(y(t+tau)) ]', 'j0' );
xlabel( 'tau/(fDts)' );
title( 'Error between J0 and autocorrelation' );

fprintf( 'The difference between Theoritical/Measured Corrleations: \n' );
fprintf( '\nMean: %15f', mean(diff) );
fprintf( '\nVariance: %15f\n', var(diff) );

%
% Make sure the real and imaginary parts are uncorrelated
%
crosscor = xcorr( real(y), imag(y) )/blockSize;

figure;
plot(ax(blockSize-10000:blockSize+10000),crosscor(blockSize-10000:blockSize+10000),ax(blockSize-10000:blockSize+10000), j0(blockSize-10000:blockSize+10000), '--' );

legend( 'E[ Imag(y(t))*Real(y(t+tau)) ]', 'j0' );
xlabel( 'tau/(fDts)' );
title( 'Crosscorrelation between real and imaginary parts' );

fprintf( '\nThe crosscorrelation between real and imaginary components:' );
fprintf( '\nMean: %15f', mean(crosscor) );
fprintf( '\nVariance: %15f\n', var(crosscor) );

