clear all;

% generator parameters
beta=10;

corThreshold = 0.01;
threshold=.0001;
Ns=25;              % odd
blockSize=5000;
Kfactor = 0;

modIndex = .4;
fD = 1;
fm = 10000;

fs = 32 * fm;
ts=1/fs;

numSamples = 100;
numTaps = 8;
td = ts * numTaps;

fDts = fD / fs;

% time axis
t = [0:ts:ts*(numSamples-1)];
tPowerDelay = [0:ts:(numTaps-1)*ts];

% input signal
s = exp( j * 2*pi*modIndex*cos(2*pi*fm*t) );

figure;
plot(s);
title( 'Input signal s(t)' );
xlabel( 'Real( s(t) )' );
ylabel( 'Imag( s(t) )' );

% Standard deviations taken from the Power Delay Profile
Ps=sqrt( ts*(-2*tPowerDelay-ts+2*td)/td^2 ); 

% the K factors
Kfactors = zeros(1,length(Ps));

% Plot the Power Delay Profile
figure;
stem(t(1:length(Ps)), Ps.^2);
title( 'Power Delay Profile' );
xlabel( 'time (s)' );
ylabel( 'Power' );

% create complex gain generator
gainGen = JakesGen( blockSize, fDts, Kfactor, Ns, threshold );
%gainGen = FWNGen( blockSize, fDts, Kfactor, beta );

myTDL = TDL( Ps, Kfactors, gainGen, corThreshold );

[myTDL, y] = process( myTDL, s );

figure;
plot( y );
title( 'Output signal y(t)' );
xlabel( 'Real( y(t) )' );
ylabel( 'Imag( y(t) )' );

